@@ -3,10 +3,7 @@ use Object::Pad;
 
 package OpenTelemetry::Propagator::TraceContext;
 
-our $VERSION = '0.015';
-
-use Log::Any;
-my $logger = Log::Any->get_logger( category =>'OpenTelemetry' );
+our $VERSION = '0.016';
 
 class OpenTelemetry::Propagator::TraceContext :does(OpenTelemetry::Propagator) {
     use experimental 'isa';
@@ -14,6 +11,7 @@ class OpenTelemetry::Propagator::TraceContext :does(OpenTelemetry::Propagator) {
     use Feature::Compat::Try;
     use URL::Encode qw( url_decode_utf8 url_encode_utf8 );
 
+    use OpenTelemetry;
     use OpenTelemetry::Propagator::TextMap;
     use OpenTelemetry::Propagator::TraceContext::TraceParent;
     use OpenTelemetry::Propagator::TraceContext::TraceState;
@@ -23,19 +21,36 @@ class OpenTelemetry::Propagator::TraceContext :does(OpenTelemetry::Propagator) {
     my $TRACE_PARENT_KEY = 'traceparent';
     my $TRACE_STATE_KEY  = 'tracestate';
 
+    use Log::Any;
+    my $logger = Log::Any->get_logger( category =>'OpenTelemetry' );
+
     method inject (
         $carrier,
         $context = OpenTelemetry::Context->current,
         $setter  = OpenTelemetry::Propagator::TextMap::SETTER
     ) {
-        my $span_context = OpenTelemetry::Trace->span_from_context($context)->context;
-        return $self unless $span_context->valid;
+        try {
+            my $span_context = OpenTelemetry::Trace
+                ->span_from_context($context)
+                ->context;
 
-        my $trace_parent = OpenTelemetry::Propagator::TraceContext::TraceParent
-            ->from_span_context($span_context);
+            return $self unless $span_context->valid;
 
-        $setter->( $carrier, $TRACE_PARENT_KEY, $trace_parent->to_string );
-        $setter->( $carrier, $TRACE_STATE_KEY,  $span_context->trace_state->to_string );
+            my $trace_parent = OpenTelemetry::Propagator::TraceContext::TraceParent
+                ->from_span_context($span_context);
+
+            $setter->( $carrier, $TRACE_PARENT_KEY, $trace_parent->to_string );
+            $setter->( $carrier, $TRACE_STATE_KEY,  $span_context->trace_state->to_string );
+        }
+        catch($e) {
+            if ( $e isa OpenTelemetry::X ) { $logger->warn($e->get_message) }
+            else {
+                OpenTelemetry->handle_error(
+                    exception => $e,
+                    message   => 'Error while injecting trace context',
+                );
+            }
+        }
 
         return $self;
     }
@@ -49,10 +64,11 @@ class OpenTelemetry::Propagator::TraceContext :does(OpenTelemetry::Propagator) {
             my $string = $getter->( $carrier, $TRACE_PARENT_KEY )
                 or return $context;
 
-            my $trace_parent = OpenTelemetry::Propagator::TraceContext::TraceParent->from_string($string);
-            my $trace_state  = OpenTelemetry::Propagator::TraceContext::TraceState->from_string(
-                $getter->( $carrier, $TRACE_STATE_KEY ) // '',
-            );
+            my $trace_parent = OpenTelemetry::Propagator::TraceContext::TraceParent
+                ->from_string($string);
+
+            my $trace_state = OpenTelemetry::Propagator::TraceContext::TraceState
+                ->from_string( $getter->( $carrier, $TRACE_STATE_KEY ) // '' );
 
             my $span_context = OpenTelemetry::Trace::SpanContext->new(
                 trace_id    => $trace_parent->trace_id,
@@ -67,8 +83,14 @@ class OpenTelemetry::Propagator::TraceContext :does(OpenTelemetry::Propagator) {
             return OpenTelemetry::Trace->context_with_span( $span, $context );
         }
         catch ($e) {
-            die $e unless $e isa OpenTelemetry::X;
-            $logger->warn($e);
+            if ( $e isa OpenTelemetry::X ) { $logger->warn($e->get_message) }
+            else {
+                OpenTelemetry->handle_error(
+                    exception => $e,
+                    message   => 'Error while extracting trace context',
+                );
+            }
+
             return $context;
         }
     }
